function [lambda_est, lambda_iter] = EM_PoissonGraph(X,theta,showstep,A)
%%This function uses the EM algorithm to find the point estimates of the
%%intensity for the Poisson Graphical Model
%%Input Arguments
%%-- X: p by n matrix represents the graphical model, where p is the number
%%      of nodes in the graph and n is the sample size
%%-- theta: scalar of nonnegative number represents the peanalized
%%          paramters
%%-- showstep: binary indictor (0/1) indicates that whether show the traces
%%          of the iterative steps. The default value is 1.
%%
%%Output Values:
%%-- lambda_est: p by p matrix contains the estimates of intensity
%%                paramters
%%-- lambda_iter: p by p by m array contains the trace of estimates over
%%                iterations, where m is the number of iteratoins


if nargin==2
    showstep=1;
end

if nargin<=3
    max_X = max(max(X));
    A = pre_compute_A(max_X);
end
    
    
    
[p,n] = size(X);

lambda_est=zeros(p,p);

logfac = [0 cumsum(A(1,:,1))];
 
%%compute initial values 
cov_est = cov(X');

lambda_iter=[];
Y_new=[];
 for i=1:p,
        for j=i:p,

                lambda_iter(i,j,1)=cov_est(i,j);
       
                lambda_iter(j,i,1)=lambda_iter(i,j,1);
        end;
 end;

t=1;
delta=1;



while delta>1e-4
    if showstep==1
        disp(sprintf('Iter %d: %.4f',t,delta));
    end
    t=t+1;
    Y_new(:,:,1:n)=0;
    
    for k=1:n,        
        expect=zeros(p,p);
        coef1=zeros(p,p);         
        for i=1:(p-1),
            for j=i+1:p,
            if sum(lambda_iter(i,:,t-1))-lambda_iter(i,j,t-1)>0 && sum(lambda_iter(j,:,t-1))-lambda_iter(i,j,t-1)>0 && lambda_iter(i,j,t-1)>0,
            coef1(i,j)=log(lambda_iter(i,j,t-1))-log(sum(lambda_iter(i,:,t-1))-lambda_iter(i,j,t-1))-log(sum(lambda_iter(j,:,t-1))-lambda_iter(i,j,t-1));

            min_X=min(X(i,k),X(j,k));
            log_p=zeros(1,min_X+1);
            prob=zeros(1,min_X+1);
            log_p(1)=-(sum(lambda_iter(i,:,t-1))+sum(lambda_iter(j,:,t-1))-lambda_iter(i,j,t-1))+X(i,k)*log(sum(lambda_iter(i,:,t-1))-lambda_iter(i,j,t-1))+X(j,k)*log(sum(lambda_iter(j,:,t-1))-lambda_iter(i,j,t-1))-logfac(1,X(i,k)+1)-logfac(1,X(j,k)+1);
            prob(1)=exp(log_p(1));
            coef2=zeros(1,min_X+1);
               

               for m=2:(min_X+1),
                  coef2(m)=A(X(i,k),X(j,k),m-1);%coef2 is the second part of coef in P(Y_ij,X_i,X_j)
                  log_p(m)=coef1(i,j)+coef2(m)+log_p(m-1);
                    prob(m)=exp(log_p(m));
                end;
                for m=1:min_X, 
                    expect(i,j)=expect(i,j)+m*prob(m+1)/sum(prob);
                end;
                Y_new(i,j,k)=expect(i,j);
               
             else
                Y_new(i,j,k)=0;
             end;
                   
             end;
        end;
    end;

        for i=1:(p-1),
            for j=i+1:p,
                lambda_iter(i,j,t)=sum(Y_new(i,j,:))/(theta+n);
                lambda_iter(j,i,t)=lambda_iter(i,j,t);
            end;
            
       
        end;
        for i=1:p,
            lambda_iter(i,i,t)=mean(X(i,:))-(theta+n)/n*sum(lambda_iter(i,:,t));
        end;
    
    delta=max(max(abs(lambda_iter(:,:,t)-lambda_iter(:,:,t-1))));
    
end

lambda_est=lambda_iter(:,:,end);
for i=2:p
    for j=1:(i-1)
        lambda_est(i,j) = 0;
    end
end
end