function [lambda_sd, lambda_lcl,lambda_ucl] = EM_PoissonGraph_Bootstrap(X,theta,nboot,showstep,A)
%%This function uses bootstrap to find make inference on the
%%intensity parameters for the Poisson Graphical Model
%%
%%Input Arguments
%%-- X: p by n matrix represents the graphical model, where p is the number
%%      of nodes in the graph and n is the sample size
%%-- theta: scalar of nonnegative number represents the peanalized
%%          paramters
%%-- nboot: number of bootstraps that are used to make inference
%%-- showstep: binary indictor (0/1) indicates that whether show the traces
%%          of the iterative steps. The default value is 1.
%%
%%Output Argumensts:
%%-- lambda_sd: p by p matrix contains the standard deviation of the 
%%              estimates of intensity paramters
%%-- lambda_lcl: p by p matrix contains the lower bound the 95% CI of the 
%%              estimates of intensity paramters
%%-- lambda_ucl: p by p matrix contains the upper bound the 95% CI of the 
%%              estimates of intensity paramters



if nargin==3
    showstep = 1;
end

if nargin<=4
    max_X = max(max(X));
    A = pre_compute_A(max_X);
end

[p,n] = size(X);
[bootstat,bootsam] = bootstrp(nboot,[],1:n);
lambda_bs = zeros(p,p,nboot);
lambda_sd = zeros(p,p);
lambda_lcl = zeros(p,p);
lambda_ucl = zeros(p,p);

for bs = 1:nboot
    if showstep==1
        disp(sprintf('Bootstrap %d',bs));
    end
    
    X_bs=X(:,bootsam(:,bs));
    lambda_bs(:,:,bs) = EM_PoissonGraph(X_bs,theta,0,A);
end

for i=1:p
    for j=i:p
        lambda_sd(i,j)=sqrt(var(lambda_bs(i,j,:)));
        lambda_lcl(i,j) = quantile(lambda_bs(i,j,:),0.025);
        lambda_ucl(i,j) = quantile(lambda_bs(i,j,:),0.975);
    end
end

end

