function [X Y]= simulate_data(lambda, n)
%% This function simulates multivariate Poisson data
%% Input arguments:
%% --lambda: p by p matrix represents the intensity parameters
%% --n: integer scalar represents the sample size
%% Output values:
%% --X: p by n matrix with each row representing a sample of multivariate 
%%      Poisson data
%% --Y: p by p by n array represents the latent independent Poisson
%%      Variables that generate the X

[p p]=size(lambda);

Y=zeros(p,p,n);
X=zeros(p,n);

for i=1:p,
    for j=i:p,
        Y(i,j,:)=poissrnd(lambda(i,j),1,n);
        Y(j,i,:)=Y(i,j,:);
    end;
end;

for i=1:p,
    X(i,:)=sum(Y(i,:,:));
end;


end